/// A Dart wrapper for the Discord API
library mercury;

import 'dart:async';
import 'dart:convert';
import 'dart:core';
import 'dart:io';
import 'package:w_transport/w_transport.dart' as transport;

// Client
part './src/session.dart';
part './src/connect_status.dart';
part './src/event_dispatcher.dart';
part './src/discord_constants.dart';
part './src/mercury_constants.dart';

// Network
part './src/net/rest_session.dart';
part './src/net/rest_response.dart';
part './src/net/rest_request_options.dart';
part './src/net/ws_session.dart';
part './src/net/ws_payload.dart';
part './src/net/ws_heartbeat.dart';

// Exceptions
part './src/errors/websocket_exception.dart';
part './src/errors/collection_exception.dart';
part './src/errors/resume_exception.dart';

// Entities
part './src/models/entity_base.dart';

// Channel & Messages
part './src/models/channel_base.dart';
part './src/models/messageable.dart';
part './src/models/message.dart';
part './src/models/guild_channel.dart';
part './src/models/guild_text_channel.dart';
part './src/models/guild_voice_channel.dart';
part './src/models/guild_category_channel.dart';
part './src/models/channel_update_properties.dart';
part './src/models/message_update_properties.dart';

// Users
part './src/models/user.dart';

// Guilds
part './src/models/guild.dart';

// Utilities
part './src/utils/embed_builder.dart';
part './src/utils/entity_collection.dart';
part './src/utils/username_validator.dart';
part './src/utils/images.dart';
part './src/utils/logger.dart';
