part of mercury;

/// A channel of any type in Discord.
abstract class ChannelBase extends EntityBase {
  /// The type of this [ChannelBase]. You can use [ChannelType] to assist with identifying.
  int type = 0;

  ChannelBase(Session session) : super(session);

  /// Updates the properties of this channel.
  Future modify(ChannelUpdateProperties props, {RequestOptions options}) async {
    var response = await session._rest.executeRoute(
        ApiEndpoints.channels.modifyChannelWithId(id, 'PATCH'),
        jsonParams: props.build({}, session));
    await _update(response.json);
    session.getCachedChannel(id).set(this);
  }

  /// Deletes this channel permanently.
  Future delete({RequestOptions options}) async {
    session.getCachedChannel(id).remove();
    await session._rest
        .executeRoute(ApiEndpoints.channels.deleteChannelWithId(id));
  }

  @override
  Future _update(Map<String, dynamic> json) async {
    type == json['type'];

    await super._update(json);
  }

  /// Converts this channel into a desired channel type.
  ///
  /// Will only work if this channel is actually of type [TDestinationType].
  TDestinationType asType<TDestinationType extends ChannelBase>() {
    return (this as TDestinationType);
  }

  /// Creates a new [ChannelBase] with the appropriate type from a JSON channel object.
  static ChannelBase createFromJson(
      Map<String, dynamic> json, Session session) {
    switch (json['type']) {
      case 0:
        // Guild text
        return new GuildTextChannel(session);
      case 1:
        // DM
        break;
      case 2:
        // Guild voice
        return new GuildVoiceChannel(session);
        break;
      case 3:
        // Group DM
        break;
      case 4:
        // Guild category
        return new GuildCategoryChannel(session);
        break;
      default:
        return null;
    }
    return null;
  }
}
