part of mercury;

/// Represents a generic Discord entity with an ID.
abstract class EntityBase {
  /// ID of this entity (usually a snowflake converted to a string)
  String id;

  /// The [Session] this entity was created by.
  Session session;

  /// A built-in [LoggerContext] extending off of the main session.
  LoggerContext logger;

  /// Updates this entity based on new JSON data.
  Future _update(Map<String, dynamic> model) async {
    id = model['id'];
  }

  /// Determines if this entity has a valid ID.
  bool get isValid => id != null;

  /// Determines if this entity has an invalid ID.
  bool get isNotValid => !isValid;

  /// Creates a blank entity.
  EntityBase(this.session) {
    logger =
        new LoggerContext.fromParent(session.logger, runtimeType.toString());
  }
}
