part of mercury;

/// Represents a Discord guild, a collection of channels and users.
class Guild extends EntityBase {
  Guild(Session client) : super(client);

  // Guild properties

  /// The name of this guild.
  String name = "";

  /// The image hash of this guilds icon.
  String iconHash = "";

  /// Gets a URL to the Discord CDN for the icon of this guild.
  String getIconUrl(String format) =>
      CDNImageHelper.guildIconEndpoint(id, iconHash, format);

  /// The image hash of this guilds splash.
  String splashHash = "";

  /// Gets a URL to the Discord CDN for the splash of this guild.
  String getSplashUrl(String format) =>
      CDNImageHelper.guildSplashEndpoint(id, splashHash, format);

  /// Whether or not this user is the owner of this guild.
  bool isOwner = null;

  /// The owner of this guild.
  User owner = null;

  // TODO: Permissions

  /// The voice region ID for this guild.
  String region = "";

  // The AFK channel (if one exists)
  // TODO: Voice channels

  /// The AFK timeout for this server (in seconds)
  int afkTimeoutSeconds = 0;

  // TODO: AFK Channels

  /// The verification level required to chat in this guild. You can use [GuildVerificationLevelHelper] to assist you.
  int verificationLevel = 0;

  /// The MFA level of this guild. You can use [MFALevelHelper] to assist you.
  int mfaLevel = 0;

  /// The Explicit Content Filter level of this guild. You can use [ExplicitContentFilterHelper] to assist you.
  int explicitContentFilterLevel = 0;

  /// The default message notification level of this guild. You can use [DefaultMessageNotificationHelper] to assist you.
  int defaultMessageNotificationLevel = 0;

  /// Whether this guild is available.
  bool isAvailable = true;

  /// Whether this guild is not available.
  bool get isNotAvailable => !isAvailable;

  /// The application ID of the guild's creator if it is bot-created.
  String applicationId = "";

  /// A list of all channels in this guild.
  _EntityCollection<ChannelBase> channels = new _EntityCollection();

  @override
  Future _update(Map<String, dynamic> json) async {
    await super._update(json);
    if (json['unavailable'] != null && json['unavailable']) {
      isAvailable = false;
      return;
    }
    isAvailable = true; // if this was false we would've returned already
    name = json['name'] ?? name;
    iconHash = json['icon'] ?? iconHash;
    splashHash = json['splash'] ?? splashHash;
    isOwner = json['owner'] ?? isOwner;
    region = json['region'] ?? region;
    afkTimeoutSeconds = json['afk_timeout'] ?? afkTimeoutSeconds;
    verificationLevel = json['verification_level'] ?? verificationLevel;
    mfaLevel = json['mfa_level'] ?? mfaLevel;
    explicitContentFilterLevel =
        json['explicit_content_filter'] ?? explicitContentFilterLevel;
    defaultMessageNotificationLevel = json['default_message_notifications'] ??
        defaultMessageNotificationLevel;

    if (json['owner_id'] != null) {
      if (session.users.containsId(json['owner_id']))
        owner = session.users.get(json['owner_id']).value;
      else {
        var ownerResponse = await session._rest
            .executeRoute(ApiEndpoints.users.getUserWithId(json['owner_id']));
        var newOwnerModel = new User(session);
        await newOwnerModel._update(ownerResponse.json);
        session.users.set(newOwnerModel);
        owner = newOwnerModel;
      }
    }

    if (json['channels'] != null) {
      for (var jsonChannel in json['channels']) {
        GuildChannel channel = ChannelBase.createFromJson(jsonChannel, session);
        await channel._update(jsonChannel);
        channel.guild = this;
        channels.set(channel);
        session.channelGuildMap[channel.id] = this.id;
      }
    }
  }
}
