part of mercury;

/// Represents a text-supported channel inside of a guild.
class GuildTextChannel extends GuildChannel implements Messageable {
  GuildTextChannel(Session client) : super(client);

  @override
  Future _update(Map<String, dynamic> json) async {
    await super._update(json);
    id = json['id'];
    type = json['type'] ?? type;
    topic = json['topic'] ?? topic;
    sortingPosition = json['position'] ?? sortingPosition;
    isNsfw = json['nsfw'] ?? isNsfw;
    lastMessageId = json['last_message_id'] ?? lastMessageId;
    parentCategoryId = json['parent_id'] ?? parentCategoryId;
  }

  // GuildTextChannel properties

  /// The position of this guild in the channel list (sorting position)
  int sortingPosition = 1;

  // TODO: Permission overwrites

  /// The topic of this channel.
  String topic = 'None';

  /// If this channel is NSFW (Not Safe For Work)
  bool isNsfw = false;

  /// The message ID of the last message sent to this channel.
  String lastMessageId = ''; // TODO: Implement this as a Message entity

  /// The ID of the parent category
  String parentCategoryId = '';

  _EntityCollection<Message> messages = new _EntityCollection();

  // TODO: Last Pin Timestamps

  /// Creates a message in this channel.
  Future<Message> createMessage(String content,
      {EmbedBuilder embed, bool tts: false, RequestOptions options}) async {
    var jsonParams = <String, dynamic>{};
    jsonParams['content'] = content;
    if (embed != null) jsonParams['embed'] = embed.build();
    jsonParams['tts'] = tts;
    var messageResponse = await session._rest.executeRoute(
        ApiEndpoints.channels.createMessage(id),
        jsonParams: jsonParams,
        options: options);
    var messageModel = new Message(session);
    await messageModel._update(messageResponse.json);
    messages.set(messageModel);
    return messageModel;
  }

  /// Gets a specific message by ID in this channel.
  Future<Message> getMessageById(String messageId,
      {RequestOptions options}) async {
    var messageResponse = await session._rest.executeRoute(
        ApiEndpoints.channels.getChannelMessage(id, messageId),
        options: options);
    var messageModel = new Message(session);
    await messageModel._update(messageResponse.json);
    messages.set(messageModel);
    return messageModel;
  }
}
