part of mercury;

/// A client to connect to Discord's REST endpoints.
class _RestSession {
  /// The owner of this REST wrapper.
  Session _client;

  /// Default constructor.
  _RestSession(this._client);

  /// Handles a request after it has been terminated due to a failure.
  Future handleRequestError(transport.RequestException err) async {
    _client.logger.log(LogLevel.Error,
        'Request errored: ' + err.message + " (${err.response.statusText})");
  }

  /// Creates and executes a JSON request based on an API route.
  Future<RestRequestResult> executeRoute(ApiRoute route,
      {Map<String, dynamic> jsonParams, RequestOptions options}) async {
    var request =
        new transport.JsonRequest(transportPlatform: _client.transportPlatform)
          ..uri = Uri.parse(route.buildRouteUrl())
          ..headers = {
            'Authorization': buildAuth(),
            'User-Agent': buildUserAgent()
          };
    if (options != null && options.auditLogReason != "") {
      request.headers['X-Audit-Log-Reason'] = options.auditLogReason;
    }
    if (route.method == "POST" ||
        route.method == "PUT" ||
        route.method == "PATCH") {
      request.body = jsonParams;
    }

    try {
      var response = await request.send(route.method);
      return new RestRequestResult.success(response);
    } on transport.RequestException catch (requestErr) {
      await handleRequestError(requestErr);
      return new RestRequestResult.failure(requestErr);
    }
  }

  /// Builds an authentication string for API requests.
  String buildAuth() => '${_getTokenPrefix()} ${_client.token}';

  /// Converts a TokenType to an adequate string for the API.
  String _getTokenPrefix() =>
      _client.tokenType.toString().replaceAll('TokenType.', '');

  /// Builds a compatible User-Agent for Discord.
  String buildUserAgent() =>
      "DiscordBot (${MercuryInfo.url}, ${MercuryInfo.version})";
}
