import 'dart:async';
import 'dart:io';

import 'package:mercury/mercury.dart';

import 'package:w_transport/vm.dart' show vmTransportPlatform;

void main() {
  var logger = new LoggerContext.root('Mercury Test Bot');
  logger.log(LogLevel.Info, 'Starting up using Mercury ${MercuryInfo.version}');
  var client = new Session(vmTransportPlatform);
  client.setToken(
      Platform.environment['CIRCUMSTELLAR_TEST_TOKEN'], TokenType.Bot);
  var msgHandlers = <String, Future Function(Message msg)>{
    'channel': (msg) async {
      var embed = new EmbedBuilder()
        ..description = 'Discord channel description'
        ..fields
            .add(new EmbedFieldBuilder('NSFW', msg.channel.isNsfw.toString()))
        ..fields.add(new EmbedFieldBuilder('Topic', msg.channel.topic))
        ..fields
            .add(new EmbedFieldBuilder('Guild Name', msg.channel.guild.name))
        ..author =
            new EmbedAuthorBuilder('Channel info for: ${msg.channel.name}')
        ..footer = new EmbedFooterBuilder('Mercury Test Bot');
      await msg.channel.createMessage('', embed: embed);
    },
    'modifychannel': (msg) async {
      var props = new ChannelUpdateProperties()..name = 'newchannel';
      await msg.channel.modify(props);
    },
    'deletechannel': (msg) async {
      await msg.channel.delete();
    },
    'greet': (msg) async {
      await msg.channel.createMessage('Hello ${msg.mentions[0].username}!');
    },
    'guildchannels': (msg) async {
      var channels = msg.channel.guild.channels;
      var sb = new StringBuffer();
      await Future.forEach(channels.getBaseMap().values, (obj) {
        var guildObj = (obj as GuildChannel);
        sb.writeln(
            '${guildObj.guild.name} -> ${guildObj.name} (type ${guildObj.runtimeType.toString()})');
      });
      if (sb.toString().isEmpty) {
        await msg.channel.createMessage("No channels in this guild(?)");
      } else
        await msg.channel.createMessage(sb.toString());
    },
    'deleteSource': (msg) async {
      await msg.delete();
    },
    'deleteAfter': (msg) async {
      var newMsg = await msg.channel
          .createMessage('I am going to delete this message soon.');
      new Timer(new Duration(seconds: 3), () async {
        await newMsg.delete();
      });
    },
    'editAfter': (msg) async {
      var newMsg = await msg.channel.createMessage('This is an old message.');
      new Timer(new Duration(seconds: 3), () async {
        var props = new MessageUpdateProperties()
          ..content = 'This is a new message.';
        await newMsg.edit(props);
      });
    },
    'chan_cache': (msg) async {
      var channelMessageCache = msg.channel.messages;
      var sb = new StringBuffer();
      await Future.forEach(channelMessageCache.values, (Message msg0) {
        sb.writeln(
            'Message | ID: ${msg0.id} | Content: `${(msg0.content ?? "No content.")}`');
      });
      var response = new StringBuffer();
      response.writeln('**This channel\'s cache is: **');
      response.writeln(
          sb.toString().isNotEmpty ? sb.toString() : 'No messages cached.');
      await msg.channel.createMessage(response.toString());
    },
    'gcache': (msg) async {
      var guildCache = msg.session.guilds.values;
      var sb = new StringBuffer();
      await Future.forEach(guildCache, (Guild guild) {
        sb.writeln(
            'Guild | ID: ${guild.id} | Name: ${guild.name} | Owner Name: ${guild.owner.username}');
      });
      await msg.channel.createMessage(sb.toString());
    }
  };
  client.dispatcher.messageCreate.listen((Message msg) async {
    var handleResult = msgHandlers.keys.firstWhere(
        (trigger) => msg.content.startsWith('!' + trigger), orElse: () {
      // If this isn't here Dart will throw a StateError
    });
    if (handleResult != null) {
      var target = msgHandlers[handleResult];
      await target(msg);
    }
  });
  client.dispatcher.ready.listen((empty) {
    logger.log(LogLevel.Info, 'Ready');
  });
  client.dispatcher.channelCreate.listen((chan) async {
    if (chan.type == ChannelType.GuildText) {
      var chan0 = (chan as GuildTextChannel);
      logger.log(LogLevel.Info, 'Channel ${chan0.name} has been created!');
    }
  });
  client.dispatcher.channelUpdate.listen((chan) async {
    if (chan.type == ChannelType.GuildText) {
      var chan1 = (chan as GuildTextChannel);
      logger.log(LogLevel.Info, 'Channel ${chan1.name} has been updated!');
    }
  });
  client.dispatcher.guildUpdate.listen((guild) async {
    logger.log(LogLevel.Info, 'Guild ${guild.name} has been updated!');
  });
  client.dispatcher.guildCreate.listen((guild) async {
    logger.log(LogLevel.Info,
        'Guild ${guild.name} has been created. Available: ${guild.isAvailable}');
  });
  client.dispatcher.messageUpdate.listen((msg) async {
    logger.log(LogLevel.Info, 'Message ${msg.id} has been updated.');
  });
  client.dispatcher.messageDelete.listen((id) async {
    logger.log(LogLevel.Info, 'Message ${id} has been deleted.');
  });
  client.connect();
}
